library ieee;
library work;

use ieee.std_logic_1164.all;
use work.emb.all;

-- e is for external, that is, the interface turning outside
-- i is for internal, that is, the interface presented to the inside
entity memory_controller is 
    port(  
            clk     : in std_logic;
            rst     : in std_logic;

            -- Interface to the external SM controller
            -- This interface mimics a SM chip
            e_addr  : in std_logic_vector(MEM_ADDR_BITS - 1 downto 0);
            e_data  : inout std_logic_vector(MEM_DATA_BITS  - 1 downto 0);
            e_cs    : in std_logic;
            e_re    : in std_logic;
            e_we    : in std_logic;
            e_wait  : out std_logic;

            -- I want to be able to do some DMA-stuff here eventually, so 
            -- I'll make another interface here \o/
            i_addr  : in std_logic_vector(USR_ADDR_BITS - 1 downto 0);
            i_data_i  : in std_logic_Vector(USR_DATA_BITS - 1 downto 0);
            i_data_o  : out std_logic_Vector(USR_DATA_BITS - 1 downto 0);
            i_re    : in std_logic;
            i_we    : in std_logic;
            i_cs    : in std_logic;
            i_wait  : out std_logic;
            
            -- Interface to internal memory
            mem_we  : out std_logic;
            mem_addr: out std_logic_vector(MEM_ADDR_BITS - 1 downto 0);
				--this goes in to this unit, from internal memory
            mem_din : in std_logic_vector(MEM_DATA_BITS - 1 downto 0);
				--this goes out of this unit, into internal memory
            mem_do  : out std_logic_vector(MEM_DATA_BITS - 1 downto 0)

            );

end entity memory_controller;


architecture rtl of memory_controller is
--    type smc_states is (idle, e_read, e_write, i_read, 
--                        i_write, e_mem_rdy, i_mem_rdy, 
--                        e_mem_write_done, i_mem_write_done, 
--                        i_cs_deassert_wait, e_cs_deassert_wait);
    type smc_states is (idle, e_read, e_mem_rdy, e_write , 
                            e_write_finished, i_read, i_mem_rdy, i_write, i_write_finished);
    signal state : smc_states;
begin
        


        memcontrol : process (clk)
        begin
        if (clk='1' and clk'event) then 
            if (rst = '0') then 
                     state <= idle;
            else  
                case state is 
                    -- Idle state, wait for chip select and control signal
                    when idle =>
                        if(e_cs = '0' and e_re = '0') then
                            state <= e_read;
                        elsif (e_cs = '0' and e_we = '0') then
                            state <= e_write;
                        elsif (i_cs = '0' and i_re = '0') then
                            state <= i_read;
                        elsif (i_cs = '0' and i_we = '0') then
                            state <= i_write;
                        else
                            state <= idle;
                        end if;
                    when e_read =>
                        state <= e_mem_rdy;
                    when e_mem_rdy =>
                        --Wait for chip select to go down (acks transaction)
                        if (e_cs = '1') then
                            state <= idle;
                        else
                            state <= e_mem_rdy;
                        end if;

                    when e_write =>
                        state <= e_write_finished;

                    when e_write_finished =>
                        state <= idle;

                    -- Internal unit
                    when i_read =>
                        state <= i_mem_rdy;

                    when i_mem_rdy =>
                        if (i_cs = '1') then
                            state <= idle;
                        else
                            state <= i_mem_rdy;
                        end if;

                   when i_write =>
                        state <= i_write_finished;

                    when i_write_finished =>
                        state <= idle;


                end case;
           end if;
       end if;
       end process;

       memcontrol_out : process (state, e_addr, i_addr, i_data_i, mem_din) 
       begin
            e_wait <= '1';
            i_wait <= '1';
            --Default values to the lollers. 
            mem_we <= '0'; --default is to read memory
            mem_addr <= (others => '0');
            mem_do <= (others => '0');

            --External interface
            e_data <= (others => 'Z');

            --Internal data output
            i_data_o <= (others => '0');

            case state is
                when idle =>
                    --nothing;
                when e_read =>
                    i_wait <= '0';
                    -- do a load from internal memory, takes this cycle to complete
                    e_wait <= '0';
                    --mem_addr <= e_addr;

                when e_mem_rdy =>
                    i_wait <= '0';
                    --the load has completed, now we'll present it to the uC
                    --note that wait goes high here (default)
                    mem_addr <= e_addr;
                    e_data <= mem_din;

                when e_write_finished =>
                    i_wait <= '0';
                    -- nothing really (e_wait = 1)

                --Write on next cycle (when entering e_mem_rdy state);
                when e_write =>
                    i_wait <= '0';
                    e_wait <= '0';
                    mem_we <= '1';
                    mem_addr <= e_addr;
                    mem_do <= e_data;

                when i_read =>
                    e_wait <= '0';
                    i_wait <= '0';
                    mem_addr <= i_addr;

                -- We idle in this state, keeping the address
                -- on the busses until the internal or external
                -- unit has acked the ready by lowering its cs.
                when i_mem_rdy =>
                    e_wait <= '0';
                    mem_addr <= i_addr;
                    i_data_o <= mem_din;
                
                when i_write =>
                    e_wait <= '0';
                    i_wait <= '0';
                    mem_we <= '1';
                    mem_addr <= i_addr;
                    mem_do <= i_data_i;

                when i_write_finished =>
                    e_wait <= '0';

            end case;
       end process;

end architecture rtl;
