library ieee;

use ieee.std_logic_1164.all;
use work.all;
use work.emb.all;


entity toplevel is 
    port (  
            main_clk : in std_logic;
            main_rst : in std_logic;
            -- Memory bus and control signals 
            mem_addr  : in std_logic_vector(MC_ADDR_BITS - 1 downto 0);
            mem_data  : inout std_logic_vector(MC_DATA_BITS  - 1 downto 0);
            mem_cs    : in std_logic;
            mem_re    : in std_logic;
            mem_we    : in std_logic;
            mem_wait  : out std_logic;

            io_port : inout std_logic_vector(63 downto 0)

            );
end entity;

architecture rtl of toplevel is
    signal usr_module_data_i  : std_logic_vector(USR_DATA_BITS - 1 downto 0);
    signal usr_module_data_o  : std_logic_vector(USR_DATA_BITS - 1 downto 0);
    signal usr_module_addr  : std_logic_vector(USR_ADDR_BITS - 1 downto 0);
    signal usr_module_re    : std_logic;
    signal usr_module_we    : std_logic;
    signal usr_module_cs    : std_logic;
    signal usr_module_wait  : std_logic;
    signal usr_module_pincfg_we : std_logic;
    signal usr_module_pincfg_oe : std_logic;
    signal usr_module_pincfg_data : std_logic_vector(15 downto 0);
    signal usr_module_pincfg_bankaddr : bank_sel_type;
    
    signal pinctrl_pinvalues : std_logic_vector(7 downto 0);

    signal memctl_we : std_logic;
    signal memctl_addr : std_logic_vector(USR_ADDR_BITS - 1 downto 0);
    signal memctl_din : std_logic_vector(USR_DATA_BITS - 1 downto 0);
    signal memctl_do : std_logic_vector(USR_DATA_BITS - 1 downto 0);

    -- synchronized signals
    signal syn_mem_cs : std_logic;    
    signal syn_mem_re : std_logic;    
    signal syn_mem_we : std_logic;    

begin
    
    memory_controller : entity work.memory_controller
    port map (
        clk => main_clk,
        rst => main_rst,
        e_addr => mem_addr,
        e_data => mem_data,
        e_cs => syn_mem_cs,
        e_re => syn_mem_re,
        e_we => syn_mem_we, 
        e_wait => mem_wait,
        i_addr => usr_module_addr,
        i_data_i => usr_module_data_o,
        i_data_o => usr_module_data_i,
        i_re => usr_module_re,
        i_we => usr_module_we,
        i_cs => usr_module_cs,
        i_wait => usr_module_wait,
        mem_we => memctl_we,
        mem_addr => memctl_addr,
        mem_din => memctl_din,
        mem_do => memctl_do
        );
   
     usr_module : entity work.usr_module 
     port map (
          clk => main_clk,
          rst => main_rst,
          data_i => usr_module_data_i,  
          data_o => usr_module_data_o,  
          addr => usr_module_addr,
          nwait => usr_module_wait,
          re => usr_module_re,
          we => usr_module_we,
          cs => usr_module_cs,

          -- Pin controller lines
          pcfg_we => usr_module_pincfg_we,
          pcfg_oe => usr_module_pincfg_oe,
          configdata => usr_module_pincfg_data,
          bank_addr => usr_module_pincfg_bankaddr,
          pinvalues => pinctrl_pinvalues

          );
  
        memory : entity work.memory
        generic map (
            width => MEM_DATA_BITS,
            length => MEM_LENGTH,
            addr_bits => MEM_ADDR_BITS
            )
        port map (
            clk => main_clk,
            we => memctl_we,
            a => memctl_addr,
            d_in => memctl_do,
            d_out => memctl_din
            );

        
        -- We need three synchronizers to clock in 
        -- the SMC control signals into the FPGA.
        cs_sync : entity synchronizer
        port map (
            clk => main_clk,
            ws => mem_cs,
            wso => syn_mem_cs ); 

        re_sync : entity synchronizer
        port map (
            clk => main_clk,
            ws => mem_re,
            wso => syn_mem_re ); 

        we_sync : entity synchronizer
        port map (
            clk => main_clk,
            ws => mem_we,
            wso => syn_mem_we ); 

    pin_controller : entity pin_control 
        port map (
            clk => main_clk,
            rst => main_rst,
            write => usr_module_pincfg_we,
            read => usr_module_pincfg_oe,
            bank_addr => usr_module_pincfg_bankaddr,
            configdata => usr_module_pincfg_data,
            pins => io_port, -- to the outside.
            pin_outdata => pinctrl_pinvalues );



end architecture rtl;
