library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.emb.all;

entity usr_module is
    port (
        clk : in std_logic;
        rst : in std_logic;
        -- Memory interface
        data : inout std_logic_vector(USR_DATA_BITS - 1 downto 0);
        addr : out std_logic_vector(USR_ADDR_BITS - 1 downto 0);
        re : out std_logic;
        we : out std_logic;
        cs : out std_logic;
        nwait : in std_logic;
        
        -- We can also create a switching matrix here.
        -- input_pins : 
        -- output_pins : 
        
        -- Pin controller interface
        pcfg_we : out std_logic;
        pcfg_oe : out std_logic;
        bank_addr : out bank_sel_type;
        configdata : out std_logic_vector(15 downto 0);
        pinvalues : in std_logic_vector(7 downto 0)

        );
end entity usr_module;


architecture rtl of usr_module is
    type lp_states is (idle, 
                        read_cmd_reg, read_cmd_reg_wait, read_cmd_reg_capture, 
                        dispatch,
                        read_pin_conf_setup, read_pin_conf_wait, read_pin_conf_capture, read_pin_conf_inc
                        ,write_pins_setup, write_pins_memctrl_setup, write_pins_wait, write_pins_finish, 
                        write_pins_inc,

                        setup_clear_cmd_reg,
                        setup_write_busy_reg,
                        setup_clear_busy_reg,
                        write_word_setup,
                        write_word_wait,
                        write_word_finish
                        
               --         mem_rd_addr_setup, mem_re_control_setup, mem_capture, 
               --         mem_wr_addr_setup, mem_we_control_setup, mem_output, mem_done
                     ); 
                        
    signal state : lp_states;

    signal current_cmd : std_logic_vector(USR_DATA_BITS - 1 downto 0);
    signal conf_words : std_logic_vector(3 downto 0) := "0000"; -- 2**4 = 16


    signal word_to_write : std_logic_vector(USR_DATA_BITS - 1 downto 0);
    signal addr_to_write : std_logic_vector(USR_ADDR_BITS - 1 downto 0);

begin

    load_and_push_statechange : process(clk, rst)
    begin
    if (clk = '1' and clk'event) then
        if (rst = '0') then
            state <= idle;
            word_to_write <= (others => '0');
            addr_to_write <= (others => '0');
            conf_words <= (others => '0');

        else 
            case state is
                when idle =>
                    state <= read_cmd_reg;

                when read_cmd_reg =>
                    state <= read_cmd_reg_wait;

                when read_cmd_reg_wait =>
                    if (nwait = '0') then
                        state <= read_cmd_reg_wait;
                    else 
                        state <= read_cmd_reg_capture;
                    end if;
                    current_cmd <= data;
                
                when read_cmd_reg_capture =>
              --      state <= setup_write_busy_reg;
                    state <= dispatch;

                when dispatch =>
                    conf_words <= "0000";
                    case current_cmd is
                        when CMD_NOOP =>
                    --        state <= setup_clear_busy_reg; -- back to idle, we have a NOOP.
                            state <= read_cmd_reg;

                        when CMD_CONF_PINS =>
                            state <= read_pin_conf_setup; -- enter the pin-config state

                        when CMD_READ_PINS =>
                            state <= write_pins_setup; -- read back pin values

                        when others =>   
                            state <= dispatch;
                    end case;

                when read_pin_conf_setup =>
                    state <= read_pin_conf_wait;

                when read_pin_conf_wait =>
                    if (nwait = '0') then
                        state <= read_pin_conf_wait;
                    else
                        state <= read_pin_conf_capture;
                    end if;

                
                when read_pin_conf_capture =>
                    state <= read_pin_conf_inc;
     
                when read_pin_conf_inc =>
                    if (unsigned(conf_words) < 7) then
                        conf_words <= std_logic_vector(
                                   unsigned(conf_words) + 1);
                        state <= read_pin_conf_setup;
                    else
--                        state <= setup_clear_cmd_reg; 
                        state <= read_cmd_reg;
                    end if;


                when write_pins_setup =>
                    state <= write_pins_memctrl_setup;

                when write_pins_memctrl_setup =>
                    state <= write_pins_wait;

                when write_pins_wait =>
                    if (nwait = '0') then
                        state <= write_pins_wait;
                    else
                        state <= write_pins_finish;
                    end if;
                when write_pins_finish =>
                    state <= write_pins_inc;

                when write_pins_inc =>
                    if (unsigned(conf_words) < 7) then
                        conf_words <= std_logic_vector(
                                   unsigned(conf_words) + 1);
                        state <= write_pins_setup;
                    else
                        state <= setup_clear_busy_reg; 
                    end if;
                

                -- States for setting up the single-command-writebacks
                --------------------------------------------------------
                when setup_clear_cmd_reg =>
                    addr_to_write <= CMD_REG;
                    word_to_write <= CMD_NOOP;
                    state <= write_word_setup;

                when setup_write_busy_reg =>
                    addr_to_write <= BUSY_REG;
                    word_to_write <= GEN_BUSY;
                    state <= write_word_setup;

                when setup_clear_busy_reg =>
                    addr_to_write <= BUSY_REG;
                    word_to_write <= NOT_BUSY;
                    state <= write_word_setup;

                -- States for reading the addr_to_write and word_to_write registers
                -- and finishing this write. It is supposed to be a general way of writing 
                -- one word back to the memory.
                --------------------------------------------------------------------------
                when write_word_setup =>
                    state <= write_word_wait;
                when write_word_wait =>
                    if (nwait = '0') then
                        state <= write_word_wait;
                    else
                        state <= write_word_finish;
                    end if;
                when write_word_finish =>
                    case word_to_write is
                        -- If we wrote the busy register, the next state is dispatch.
                        when GEN_BUSY =>
                            state <= dispatch;
                        -- Clear the command register.
                        when NOT_BUSY =>
                            state <= setup_clear_cmd_reg; 
                        -- Just to be explicit; if we come here we are finished with the cmd.
                        when CMD_NOOP =>
                            state <= read_cmd_reg;
                        when others =>
                            state <= read_cmd_reg;
                    end case;

            end case;
        end if; --rst if
    end if; --clk if
    end process;


    bank_addr <= conf_words;

    load_and_push_output : process(state)
    begin

        --Default outputs

        configdata <= (others => '0');
        data <= (others => 'Z');
        addr <= (others => '0');
        re <= '1';
        we <= '1';
        cs <= '1';
      
        --Pin config mofule
        pcfg_we <= '1';
        pcfg_oe <= '1';

        case state is
            when idle =>
                --data_out(7 downto 0) <= pinvalues;

            when read_cmd_reg =>
                addr <= CMD_REG;  
                re <= '0';
                cs <= '0';

           when read_cmd_reg_wait => 
                addr <= CMD_REG; 
                re <= '0';
                cs <= '0';

            when read_cmd_reg_capture =>
                cs <= '0';
                re <= '0';

            when dispatch =>

            when read_pin_conf_setup =>
                cs <= '0';
                re <= '0';
                addr <= std_logic_vector(
                          unsigned(CONF_WORDS_BASE) + 
                          unsigned(conf_words));

            when read_pin_conf_wait =>
                cs <= '0';
                re <= '0';
                addr <= std_logic_vector(
                          unsigned(CONF_WORDS_BASE) + 
                          unsigned(conf_words));
 
            when read_pin_conf_capture =>
                -- Push the config to the banks
                cs <= '0';
                re <= '0';
                pcfg_we <= '0';
                addr <= std_logic_vector(
                          unsigned(CONF_WORDS_BASE) + 
                          unsigned(conf_words));
                configdata <= data;

           
            when read_pin_conf_inc =>
                
            -- Read back the pinvalues stored in the banks. First setup
            -- the bank address, and on the next flank, there will be data 
            -- on the pincontrollers output port.
            when write_pins_setup =>
                pcfg_oe <= '0';
          
           -- The pincontroller now has provided the data, and
           -- we can push them onto our bus.
           when write_pins_memctrl_setup =>
                pcfg_oe <= '0';

                -- Setup the outputs to the SMC.
                addr <= std_logic_vector(
                          unsigned(PINVALS_BASE) + 
                          unsigned(conf_words));
                data <= X"00" & pinvalues;
 

            when write_pins_wait =>
                pcfg_oe <= '0';

                addr <= std_logic_vector(
                          unsigned(PINVALS_BASE) + 
                          unsigned(conf_words));
                cs <= '0';
                we <= '0';
                data <= X"00" & pinvalues;
                
            when write_pins_finish =>
                -- Chip select deassert.
            when write_pins_inc =>
            
            
            when setup_clear_cmd_reg =>
            when setup_write_busy_reg =>
            when setup_clear_busy_reg =>
            when write_word_setup =>
                cs <= '0';
                we <= '0';
                addr <= addr_to_write;
                data <= word_to_write;

            when write_word_wait =>
                cs <= '0';
                we <= '0';
                addr <= addr_to_write;
                data <= word_to_write;
 
            when write_word_finish =>
                -- chirp chirp
 
            
        end case; 
    end process;
end architecture rtl;
