library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.emb.all;

entity usr_module is
    port (
        clk : in std_logic;
        rst : in std_logic;
        -- Memory interface
        data_i : in std_logic_vector(USR_DATA_BITS - 1 downto 0);
        data_o : out std_logic_vector(USR_DATA_BITS - 1 downto 0);
        addr : out std_logic_vector(USR_ADDR_BITS - 1 downto 0);
        re : out std_logic;
        we : out std_logic;
        cs : out std_logic;
        nwait : in std_logic;
        
        -- We can also create a switching matrix here.
        -- input_pins : 
        -- output_pins : 
        
        -- Pin controller interface
        pcfg_we : out std_logic;
        pcfg_oe : out std_logic;
        bank_addr : out bank_sel_type;
        configdata : out std_logic_vector(15 downto 0);
        pinvalues : in std_logic_vector(7 downto 0)

        );
end entity usr_module;


architecture rtl of usr_module is
    type lp_states is (idle, 
                        read_cmd_reg, read_cmd_reg_wait, read_cmd_reg_capture, 
--                        dispatch,
                        read_pin_conf_setup, read_pin_conf_wait, read_pin_conf_capture, read_pin_conf_inc
                        ,write_pins_setup, write_pins_memctrl_setup, write_pins_wait, write_pins_finish, 
                        write_pins_inc,
                        clear_cmd_reg_setup,
                        clear_cmd_reg_wait,
                        clear_cmd_reg_finish
                        
                     ); 
                        
    signal state : lp_states;

--    signal current_cmd : std_logic_vector(USR_DATA_BITS - 1 downto 0);
    signal conf_words : bank_sel_type; --std_logic_vector(2 downto 0) := "000"; -- 2**4 = 16



begin
    bank_addr <= conf_words;

    load_and_push_statechange : process(clk, rst)
    begin
    if (clk = '1' and clk'event) then
        if (rst = '0') then
            state <= idle;
    		conf_words <= "0000";
        else 
            case state is
                when idle =>
                    state <= read_cmd_reg;

                when read_cmd_reg =>
                    state <= read_cmd_reg_wait;

                when read_cmd_reg_wait =>
                    if (nwait = '0') then
                        state <= read_cmd_reg_wait;
                    else 
                        state <= read_cmd_reg_capture;
                    end if;
                
                when read_cmd_reg_capture =>
--                    state <= dispatch;

--                when dispatch =>
                    case data_i is
                        when CMD_NOOP =>
                            state <= idle;

                        when CMD_CONF_PINS =>
                            state <= read_pin_conf_setup; -- enter the pin-config state

                        when CMD_READ_PINS =>
                            state <= write_pins_setup; -- read back pin values

                        when others =>   
                            state <= idle;

                    end case;

                when read_pin_conf_setup =>
                    state <= read_pin_conf_wait;

                when read_pin_conf_wait =>
                    if (nwait = '0') then
                        state <= read_pin_conf_wait;
                    else
                        state <= read_pin_conf_capture;
                    end if;

                
                when read_pin_conf_capture =>
                    state <= read_pin_conf_inc;
     
                when read_pin_conf_inc =>
                    if (conf_words = "1000") then
                        state <= clear_cmd_reg_setup;
                    else
        			conf_words <= std_logic_vector(
                                   	unsigned(conf_words) + 1);
                        state <= read_pin_conf_setup;

                    end if;


                when write_pins_setup =>
                    state <= write_pins_memctrl_setup;

                when write_pins_memctrl_setup =>
                    state <= write_pins_wait;

                when write_pins_wait =>
                    if (nwait = '0') then
                        state <= write_pins_wait;
                    else
                        state <= write_pins_finish;
                    end if;

                when write_pins_finish =>
                    state <= write_pins_inc;

                when write_pins_inc =>
                    if (conf_words = "1000") then
                        state <= clear_cmd_reg_setup; 
                    else
        			conf_words <= std_logic_vector(
                                   unsigned(conf_words) + 1);
                        state <= write_pins_setup;
                    end if;
                
                when clear_cmd_reg_setup =>
                    conf_words <= "0000";
                    state <= clear_cmd_reg_wait;

                when clear_cmd_reg_wait =>
                    if (nwait = '0') then
                        state <= clear_cmd_reg_wait;
                    else
                        state <= clear_cmd_reg_finish;
                    end if;
                
                when clear_cmd_reg_finish =>
                    state <= read_cmd_reg;

            end case;
        end if; --rst if
    end if; --clk if
    end process;



    load_and_push_output : process(state,conf_words,pinvalues, data_i)
    begin

        --Default outputs

        configdata <= (others => '0');
        data_o <= (others => '0');
        addr <= (others => '0');
        re <= '1';
        we <= '1';
        cs <= '1';
      
        --Pin config mofule
        pcfg_we <= '1';
        pcfg_oe <= '1';
	--current_cmd <= (others => '0');

        case state is
            when idle =>
                --data_out(7 downto 0) <= pinvalues;

            when read_cmd_reg =>
                addr <= CMD_REG;  
                re <= '0';
                cs <= '0';
	    when read_cmd_reg_wait => 
                addr <= CMD_REG; 
                re <= '0';
                cs <= '0';
          --      current_cmd <= data_i;

            when read_cmd_reg_capture =>
                
--            when dispatch =>

            when read_pin_conf_setup =>
                cs <= '0';
                re <= '0';
                addr <= std_logic_vector(
                          unsigned(CONF_WORDS_BASE) + 
                          unsigned(conf_words));

            when read_pin_conf_wait =>
                cs <= '0';
                re <= '0';
                addr <= std_logic_vector(
                          unsigned(CONF_WORDS_BASE) + 
                          unsigned(conf_words));
                configdata <= data_i;
 
            when read_pin_conf_capture =>
                -- Push the config to the banks
                pcfg_we <= '0';
                addr <= std_logic_vector(
                          unsigned(CONF_WORDS_BASE) + 
                          unsigned(conf_words));
                configdata <= data_i;

           
            when read_pin_conf_inc =>
                
            -- Read back the pinvalues stored in the banks. First setup
            -- the bank address, and on the next flank, there will be data 
            -- on the pincontrollers output port.
            when write_pins_setup =>
                pcfg_oe <= '0';
          
           -- The pincontroller now has provided the data, and
           -- we can push them onto our bus.
           when write_pins_memctrl_setup =>
                pcfg_oe <= '0';
                cs <= '0';
                we <= '0';

                -- Setup the outputs to the SMC.
                addr <= std_logic_vector(
                          unsigned(PINVALS_BASE) + 
                          unsigned(conf_words));
--                data_o <= X"00" & pinvalues;
 

            when write_pins_wait =>
                pcfg_oe <= '0';

                addr <= std_logic_vector(
                          unsigned(PINVALS_BASE) + 
                          unsigned(conf_words));
                cs <= '0';
                we <= '0';
                data_o <= X"00" & pinvalues;
                
            when write_pins_finish =>
                -- Chip select deassert.
            when write_pins_inc =>
 

            when clear_cmd_reg_setup =>
                cs <= '0';
                we <= '0';
                addr <= CMD_REG;

            when clear_cmd_reg_wait =>
                 cs <= '0';
                we <= '0';
                addr <= CMD_REG;
                data_o <= CMD_NOOP;
               	--current_cmd <= CMD_NOOP;
            
            when clear_cmd_reg_finish =>
	    	
                -- Chip select and write enable deassert.


        end case; 
    end process;
end architecture rtl;
