#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include "emb.h"
#include "datalink.h"
#include <stdint.h>
#include <arpa/inet.h>
#include "pin.h"
#include <time.h>
#include "config.h"

int setPattern(pattern_t * pattern, pinconfig_t * conf)
{
//    printf("Sending pattern\n");
    uint8_t * data = config_to_emb_data(pattern, conf);
    evoframe_t frame;
    evoframe_header_t * head;
   
    head = create_frame_header(0x42, conf->npins/PINS_PER_BYTE, 0, CMD_SEND_PATTERN); 
 //   printf("type: %d\n", head.type);
    frame.header = head;
    frame.data = data;
    
    send_frame(&frame, conf->npins/PINS_PER_BYTE);
    free(head);

    return 0;
}


pattern_t * readPattern(pinconfig_t * config)
{
    //Send command to fpga
    evoframe_t frame;
    evoframe_header_t * head;
    head = create_frame_header(0x42, 0, 0, CMD_READ_PATTERN);
    frame.header = head;
    frame.data = NULL;
    
    send_frame(&frame, 0);
    free(head);
    int response;
    
 //   printf("Waiting for response from UC:\n");
    if((response = getResponse()) != RESP_OK) {
        fprintf(stderr, "readPattern(): send_frame() failed with code %d\n", response);
    }

    //Wait for data back.
    evoframe_t * frame_ret;
    frame_ret = recv_frame();

    //Mask out the output pins according to the config. 
    //get_input_bytes returns allocated data.
    int i;
    uint8_t * retdata = get_input_bytes(frame_ret->data, config);

    //Done with the frame data.
    free(frame_ret->data);
  
    pattern_t * ret = malloc(sizeof(pattern_t));
    ret->nbytes = (int)ceil(config->ninput/(double)8);
    ret->data = retdata; 
    
    return ret;
}

pattern_t * setWaitRead(pattern_t * pattern, pinconfig_t * config, int wait)
{
    setPattern(pattern, config);
    usleep(1000*wait);
    return readPattern(config);
}


int getResponse()
{
    evoframe_t * ret = recv_frame();

    if(ret->header->type != CMD_RESPONSE) {
        fprintf(stderr, "Expected response from EMB, got %d-type frame\n", ret->header->type);
    } else {
        //printf("Response ok!\n");
    }

    uint32_t response = ntohl(*(uint32_t*)ret->data);
    free(ret->data);

    return (int)response;
}


int getNumBytesInput(pinconfig_t * conf) 
{   
    int i;
    int n;
    for(i = 0; i < conf->npins; i++) {
        if(conf->pins[i].mode == PIN_IN) {
            n++;
        }
    }
    return n/8;
}

int getNumBytesOutput(pinconfig_t * conf) 
{   
    int i;
    int n;
    for(i = 0; i < conf->npins; i++) {
        if(conf->pins[i].mode == PIN_OUT) {
            n++;
        }
    }
    return n/8;
}


int allocatePatterns(int n, pinconfig_t * config) 
{
    uint32_t data = htonl(n);
    evoframe_t * cmd = create_frame(CMD_ALLOC, (uint8_t*)&data, sizeof(int) );
    send_frame(cmd, sizeof(int));

    free(cmd); 

    return getResponse();
}

int storePattern(pattern_t * pattern, int n, pinconfig_t * conf)
{
    uint8_t * data = config_to_emb_data(pattern, conf);
    int i;

    evoframe_t * frame = create_frame(CMD_STORE_VECTOR, data, CFG_VECTOR_BYTES);
    frame->header->vector = htonl(n);

    send_frame(frame, CFG_VECTOR_BYTES);

    return getResponse();
}

int storeRndPattern(int num, pinconfig_t * conf)
{
    //Create a random pattern and store it using storePattern();
    int pattern_bytes = (int)ceil(conf->noutput/(double)8);
    pattern_t * pat = generate_random_pattern(pattern_bytes);

    return storePattern(pat, num, conf);
}

pattern_t * applyPattern(int n, pinconfig_t * config, int wait)
{
    evoframe_t * frame = create_frame(CMD_APPLY_VECTOR, NULL, 0);
    frame->header->vector = htonl(n);
    
    uint32_t data = htonl(wait);
    frame->data = (uint8_t *)&data;
    
    send_frame(frame, 0);
    if(getResponse() != 0) {
        fprintf(stderr, "applyPattern failed board-side\n");
    } else {
        printf("applyPattern succeeded\n");
    }

    frame = create_frame(CMD_READ_VECTOR, NULL, 0);
    frame->header->vector = htonl(n);
    send_frame(frame, 0);
   
    printf("reading vector\n");
    return loadPattern(n, config);
}

int detectStabilityPattern(uint32_t samplePeriod, uint32_t stabilityPeriod, 
                           uint32_t timeout, pattern_t * pat, 
                           pinconfig_t * config)
{

    //First we set the pattern.
    printf("setting pattern in detectStability\n");
    setPattern(pat, config);
   
    printf("ok, done. next send command\n");
    //Then, we send a command.
    uint32_t data[4];
    data[0] = htonl(samplePeriod);
    data[1] = htonl(stabilityPeriod);
    data[2] = htonl(timeout);
    data[3] = 0x88;

    evoframe_t * frame = create_frame(CMD_DETECT_STABILITY, (uint8_t*)data, 4 * sizeof(uint32_t));

    //ship command to the uC
    send_frame(frame, 4 * sizeof(uint32_t));
     
    return getResponse();
}


pattern_t * loadPattern(int n, pinconfig_t * config)
{
    evoframe_t * frame = create_frame(CMD_READ_VECTOR, NULL, 0);
    frame->header->vector = htonl(n);
    
    send_frame(frame, 0);

    //We will now get a result.
    frame = recv_frame();
    int i=0;
    uint8_t * retdata = get_input_bytes(frame->data, config);
    free(frame->data);

    pattern_t * ret = malloc(sizeof(pattern_t));
    ret->data = retdata; 
    ret->nbytes = (int)ceil(config->ninput/(double)8);

    return ret;
}

pattern_t ** applyAllPatterns(int wait, int n, pinconfig_t * cfg)
{
    uint32_t data = htonl(wait);
    evoframe_t * cmd = create_frame(CMD_APPLY_VECTOR_SET, (uint8_t*)&data, sizeof(uint32_t) );
    send_frame(cmd, sizeof(int));

    //Allocate room for return vectors 
    int pattern_bytes = (int)ceil(cfg->ninput/(double)8);

    pattern_t ** ret = malloc(sizeof(pattern_t *) * n);
    int i;
    for(i = 0; i < n; i++) {
        (ret[i]) = loadPattern(i, cfg);
    }
    
    return ret;
}

/* Generate a random pattern */
pattern_t * generate_random_pattern(int bytes)
{
    int i;
    pattern_t * pat = (pattern_t*)malloc(sizeof(pattern_t *));
    pat->nbytes = bytes;
    pat->data = malloc(bytes);

    struct timespec ts;
    clock_gettime(CLOCK_MONOTONIC, &ts);
    srand(ts.tv_nsec);
 
    //seed it with time.

    for(i = 0; i < bytes; i++) {
        pat->data[i] = rand(); 
    }
    return pat;
}

void free_patterns(pattern_t ** nbs, int n) 
{
    //Free all neighbours, one by one
    int i;
    for(i = 0; i < n; i++) {
        free(nbs[i]);            
    }
    //Free list itself
    free(nbs);
}


